# Creating the core event scheduler
set ns [new Simulator -multicast on]

$ns color 1 red
$ns color 2 blue

# Creating 11 nodes
for {set i 0} {$i < 11} {incr i} {
    set n($i) [$ns node]
}

# Outputs nam traces
set nf [open out.nam w]
$ns namtrace-all $nf

# Creating 10 links
# All with bandwidth 10Mb, delay 10ms, and DropTail queue
# Except the bottleneck link with bw 1.5Mb, delay 20ms, and RED queue
$ns duplex-link $n(0) $n(3) 10Mb 10ms DropTail
$ns duplex-link $n(1) $n(3) 10Mb 10ms DropTail
$ns duplex-link $n(2) $n(3) 10Mb 10ms DropTail
$ns duplex-link $n(3) $n(4) 1.5Mb 20ms RED
$ns duplex-link $n(4) $n(5) 10Mb 10ms DropTail
$ns duplex-link $n(4) $n(6) 10Mb 10ms DropTail
$ns duplex-link $n(5) $n(7) 10Mb 10ms DropTail
$ns duplex-link $n(5) $n(8) 10Mb 10ms DropTail
$ns duplex-link $n(6) $n(9) 10Mb 10ms DropTail
$ns duplex-link $n(6) $n(10) 10Mb 10ms DropTail

# Set queue limit (buffer size) to 5 packets in the bottleneck link
$ns queue-limit $n(3) $n(4) 5
$ns queue-limit $n(4) $n(3) 5

# Set link orientation for nam 
$ns duplex-link-op $n(0) $n(3) orient right-down
$ns duplex-link-op $n(1) $n(3) orient right
$ns duplex-link-op $n(2) $n(3) orient right-up
$ns duplex-link-op $n(3) $n(4) orient right
$ns duplex-link-op $n(4) $n(5) orient right-up
$ns duplex-link-op $n(4) $n(6) orient right-down
$ns duplex-link-op $n(5) $n(7) orient right-up
$ns duplex-link-op $n(5) $n(8) orient right
$ns duplex-link-op $n(6) $n(9) orient right
$ns duplex-link-op $n(6) $n(10) orient right-down

# Set queue position for nam 
$ns duplex-link-op $n(3) $n(4) queuePos -0.5
$ns duplex-link-op $n(4) $n(3) queuePos -0.5

# Set multicast routing to dense mode
$ns mrtproto DM {}

# Create a TCP connection from node 9 to node 1
set tcp [new Agent/TCP]
set sink [new Agent/TCPSink]
$ns attach-agent $n(9) $tcp
$ns attach-agent $n(1) $sink
$ns connect $tcp $sink
$tcp set class_ 1
$tcp set window_ 20
# Create an FTP source and attach it to the TCP connection
set ftp [new Application/FTP]
$ftp attach-agent $tcp

# Create a UDP connection from node 7 to a multicast group
set group [Node allocaddr]
set udp [new Agent/UDP]
$ns attach-agent $n(7) $udp
$udp set dst_addr_ $group
$udp set dst_port_ 0
$udp set class_ 2

# Create a CBR source and attach it to the UDP connection
set cbr [new Application/Traffic/CBR]
$cbr attach-agent $udp

# Create dummy receivers and attach them to the receiving nodes
set rcvr0 [new Agent/Null]
$ns attach-agent $n(0) $rcvr0
set rcvr1 [new Agent/Null]
$ns attach-agent $n(2) $rcvr1

# Schedule events to happen
$ns at 0.0 "$n(0) join-group $rcvr0 $group"
$ns at 0.0 "$n(2) join-group $rcvr1 $group"
$ns at 0.2 "$cbr start"
$ns at 0.2 "$ftp start"
$ns at 1.5 "finish"

# A finish proc to flush traces and out call nam
proc finish {} {
        global ns nf
        $ns flush-trace
        close $nf

        puts "running nam..."
        exec nam out.nam &
        exit 0
}

# Start the event scheduler
$ns run
